// -*- C++ -*-
/*!
* @file  Mixture.cpp
* @brief Mixture of Saliency Models
* @date $Date$
*
* $Id$
*/

#include "Mixture.h"
#include <iostream>


// Module specification
// <rtc-template block="module_spec">
static const char* mixture_spec[] =
{
	"implementation_id", "Mixture",
	"type_name",         "Mixture",
	"description",       "Mixture of Saliency Models",
	"version",           "1.0.0",
	"vendor",            "Nakamura",
	"category",          "Saliency_Model",
	"activity_type",     "PERIODIC",
	"kind",              "DataFlowComponent",
	"max_instance",      "1",
	"language",          "C++",
	"lang_type",         "compile",
	// Configuration variables
	"conf.default.view", "1:ON",
	"conf.default.BMS", "0.31",
	"conf.default.CovSal", "0.24",
	"conf.default.Judd", "0.45",
	// Widget
	"conf.__widget__.view", "radio",
	"conf.__widget__.BMS", "text",
	"conf.__widget__.CovSal", "text",
	"conf.__widget__.Judd", "text",
	// Constraints
	"conf.__constraints__.view", "(1:ON,0:OFF)",
	""
};
// </rtc-template>

/*!
* @brief constructor
* @param manager Maneger Object
*/
Mixture::Mixture(RTC::Manager* manager)
	// <rtc-template block="initializer">
	: RTC::DataFlowComponentBase(manager),
	m_CovSalIn("CovSal", m_CovSal),
	m_JuddIn("Judd", m_Judd),
	m_BMSIn("BMS", m_BMS),
	m_saliencyMapOut("saliencyMap", m_saliencyMap)

	// </rtc-template>
{
	StepwiseExecutionContextInit(m_pManager);
}

/*!
* @brief destructor
*/
Mixture::~Mixture()
{
}



RTC::ReturnCode_t Mixture::onInitialize()
{
	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
	// Set InPort buffers
	addInPort("CovSal", m_CovSalIn);
	addInPort("Judd", m_JuddIn);
	addInPort("BMS", m_BMSIn);

	// Set OutPort buffer
	addOutPort("saliencyMap", m_saliencyMapOut);

	// Set service provider to Ports

	// Set service consumers to Ports

	// Set CORBA Service Ports

	// </rtc-template>

	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("view", m_view, "1:ON");
	bindParameter("BMS", m_W_BMS, "0.31");
	bindParameter("CovSal", m_W_CovSal, "0.24");
	bindParameter("Judd", m_W_Judd, "0.45");
	// </rtc-template>

	m_saliencyMapOut.initialize();

	prevflg = 0;

	return RTC::RTC_OK;
}


RTC::ReturnCode_t Mixture::onFinalize()
{
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Mixture::onStartup(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Mixture::onShutdown(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Mixture::onActivated(RTC::UniqueId ec_id)
{
	return RTC::RTC_OK;
}


RTC::ReturnCode_t Mixture::onDeactivated(RTC::UniqueId ec_id)
{
	switch_viewer(saliencyImageMat, m_saliencyMapOut.getName(), prevflg, 0);

	BMSMat.release();
	CovSalMat.release();
	JUDDMat.release();
	saliencyImageMat.release();

	while(m_CovSalIn.isNew())
		m_CovSalIn.read();

	while(m_JuddIn.isNew())
		m_JuddIn.read();

	while(m_BMSIn.isNew())
		m_BMSIn.read();


	return RTC::RTC_OK;
}


RTC::ReturnCode_t Mixture::onExecute(RTC::UniqueId ec_id)
{
	if (!m_JuddIn.isNew() || !m_BMSIn.isNew() || !m_CovSalIn.isNew())
	{
		STEPWISE_NOTIFY_NO_DATA();
		return RTC::RTC_OK;
	}

	m_JuddIn.read(JUDDMat);
	m_BMSIn.read(BMSMat);
	m_CovSalIn.read(CovSalMat);

	JUDDMat.convertTo(JUDDMat,CV_64F);
	BMSMat.convertTo(BMSMat,CV_64F);
	CovSalMat.convertTo(CovSalMat,CV_64F);

	saliencyImageMat = ( (m_W_BMS * BMSMat) +  (m_W_CovSal * CovSalMat) + (m_W_Judd * JUDDMat) ) / (m_W_BMS + m_W_CovSal + m_W_Judd);
	m_saliencyMapOut.write(saliencyImageMat);

	saliencyImageMat.convertTo(viewImage, CV_8U, 255, 0);
	switch_viewer(viewImage, m_saliencyMapOut.getName(), prevflg, m_view);
	

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Mixture::onAborting(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Mixture::onError(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Mixture::onReset(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Mixture::onStateUpdate(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Mixture::onRateChanged(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/



extern "C"
{

	void MixtureInit(RTC::Manager* manager)
	{
		coil::Properties profile(mixture_spec);
		manager->registerFactory(profile,
			RTC::Create<Mixture>,
			RTC::Delete<Mixture>);
	}

};


void Mixture::switch_viewer(const cv::Mat& src, const std::string name, int& prev, const int curr)
{
	if(curr != prev)	//flg changed
	{	
		if(prev == 1)
		{		
			prev = 0;
			std::cout << name << " viewer OFF" << std::endl;
			cv::destroyWindow(name);
		}
		else
		{		
			prev = 1;
			std::cout << name << " viewer ON" << std::endl;
			cv::namedWindow(name, CV_WINDOW_AUTOSIZE);
		}
	}	
	if(curr == 1){
#if (!defined WIN32) || (!defined WIN64)
		cvStartWindowThread();
#endif
		cv::imshow(name,src);
		cv::waitKey(1);
	}
}
